import json
import httpx
from .config import *
from bs4 import BeautifulSoup


# 将自定义的类转化为字典，dumps方法使用
def obj_to_dict(obj):
    d = {'__class__': obj.__class__.__name__, '__module__': obj.__module__}
    d.update(obj.__dict__)
    return d


# 将字典转化为自定义的类，loads方法使用
def dict_to_obj(d):
    if '__class__' in d:
        class_name = d.pop('__class__')
        module_name = d.pop('__module__')
        module = __import__(module_name)
        class_ = getattr(module, class_name)
        args = dict((key.encode('ascii'), value) for key, value in d.items())
        instance = class_(**args)
    else:
        instance = d
    return instance


def check_captcha(text: str) -> bool:
    """验证码"""
    text = text.strip()
    if len(text) == 4 and text.isalnum():
        return True
    return False


def add_data(data: list):
    """将当前获取的房间数据放于库内"""
    hs = hash(str(data[0] + str(data[2])))
    config.MatchDict[hs] = data


def check_data(data: list) -> bool:
    """检测当前房间数据是否存在于库内"""
    hs = hash(str(data[0] + str(data[2])))
    if hs in config.MatchDict:
        return True
    else:
        return False


async def sava_cookies(res):
    """保存cookies"""
    dic = {}
    for key, value in res.cookies.items():
        dic[key] = value
    config.LoginCookies = dic
    cookies_json = json.dumps(dic)
    with open(file_cookies, 'w+') as f:
        f.write(cookies_json)


async def get_cookies() -> dict:
    """从config或本地文件中获取保存的cookies"""
    if len(config.LoginCookies) != 0:
        return config.LoginCookies
    elif file_cookies.exists():
        with open(file_cookies, 'r') as f:
            cookies = f.read().strip()
            if cookies != '':
                return json.loads(cookies)
    return {}


async def save_userinfo(data: list):
    """将用户数据保存在本地"""
    with open(file_userinfo, 'w+') as f:
        f.write(json.dumps(data, default=obj_to_dict))


async def get_userinfo() -> list:
    """从本地文件中获取备份的用户数据"""
    if file_userinfo.exists():
        with open(file_userinfo, 'r') as f:
            userinfo = f.read().strip()
            if userinfo != '':
                return json.loads(userinfo, default=dict_to_obj)
    return []


async def request_login(captcha):
    login_cookies = await get_cookies()
    url = "https://ctf.bugku.com:443/login/check.html"
    data = {"username": config.bugku_username,
            "password": config.bugku_password, "vcode": captcha, "autologin": "1"}
    login_headers = {"Sec-Ch-Ua": "\"Chromium\";v=\"92\", \" Not A;Brand\";v=\"99\", \"Google Chrome\";v=\"92\"",
                     "Accept": "*/*", "X-Requested-With": "XMLHttpRequest", "Sec-Ch-Ua-Mobile": "?0",
                     "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36",
                     "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
                     "Origin": "https://ctf.bugku.com", "Sec-Fetch-Site": "same-origin", "Sec-Fetch-Mode": "cors",
                     "Sec-Fetch-Dest": "empty", "Referer": "https://ctf.bugku.com/login",
                     "Accept-Encoding": "gzip, deflate", "Accept-Language": "zh-CN,zh;q=0.9", "Connection": "close"}
    async with httpx.AsyncClient() as client:
        res = await client.post(url, data=data, cookies=login_cookies, headers=login_headers)
    return res


async def request_detect():
    detect_url = "https://ctf.bugku.com:443/pvp.html"
    cookies = await get_cookies()
    headers = {"Cache-Control": "max-age=0",
               "Sec-Ch-Ua": "\"Chromium\";v=\"92\", \" Not A;Brand\";v=\"99\", \"Google Chrome\";v=\"92\"",
               "Sec-Ch-Ua-Mobile": "?0", "Upgrade-Insecure-Requests": "1",
               "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36",
               "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
               "Sec-Fetch-Site": "same-origin", "Sec-Fetch-Mode": "navigate", "Sec-Fetch-User": "?1",
               "Sec-Fetch-Dest": "document", "Referer": "https://ctf.bugku.com/pvp.html",
               "Accept-Encoding": "gzip, deflate", "Accept-Language": "zh-CN,zh;q=0.9", "Connection": "close"}
    async with httpx.AsyncClient() as client:
        res = await client.get(detect_url, cookies=cookies, headers=headers)
    return res


async def get_captcha():
    """请求验证码文件放入res文件夹并保存当前cookies"""
    url = "https://ctf.bugku.com/captcha.html"
    async with httpx.AsyncClient() as client:
        res = await client.get(url)
        with open(img_captcha, 'wb+') as f:
            f.write(res.content)
    await sava_cookies(res)


async def match_data_solve(res) -> list:
    """处理返回值"""
    soup = BeautifulSoup(res.text, 'lxml')
    soup_tbody = soup.find('tbody')
    match_list = []
    for i in soup_tbody.children:
        if str(i).strip() == '':
            continue
        status = i.td.span.string
        if status == '即将开始':
            per_list = []
            for j in i.children:
                s = j.string
                if s is None or s == '\n' or s == ' ':
                    continue
                per_list.append(s.strip())
            match_list.append(per_list)
        elif status == "已经结束":
            break
    return match_list


async def check_in():
    url = "https://ctf.bugku.com:443/user/checkin"
    cookies = get_cookies()
    headers = {"Sec-Ch-Ua": "\"Chromium\";v=\"92\", \" Not A;Brand\";v=\"99\", \"Google Chrome\";v=\"92\"",
               "Accept": "*/*", "X-Csrf-Token": "0f564f2a5d35352abd95f0602dacc520", "Sec-Ch-Ua-Mobile": "?0",
               "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.107 Safari/537.36",
               "X-Requested-With": "XMLHttpRequest", "Sec-Fetch-Site": "same-origin", "Sec-Fetch-Mode": "cors",
               "Sec-Fetch-Dest": "empty", "Referer": "https://ctf.bugku.com/", "Accept-Encoding": "gzip, deflate",
               "Accept-Language": "zh-CN,zh;q=0.9", "Connection": "close"}
    async with httpx.AsyncClient() as client:
        res = await client.get(url, cookies=cookies, headers=headers)
    return res



import os
from nonebot import get_driver
from pydantic import BaseSettings
from pathlib import Path
from dataclasses import dataclass


class Config(BaseSettings):
    InitStatus: bool = False
    LoginStatus: bool = False
    LoginCookies: dict = {}
    MatchDict: dict = []
    MatchList: list = []
    UserInfo: list = []
    bugku_username: str
    bugku_password: str

    class Config:
        extra = "ignore"


@dataclass
class User:
    user: str
    user_type: str


global_config = get_driver().config
config = Config(**global_config.dict())
current_path = Path.cwd()
res_path = (current_path.parent / 'res').resolve()
img_captcha = res_path / 'captcha.png'
img_test = res_path / '1.jpg'
file_cookies = res_path / 'cookies.json'
file_userinfo = res_path / 'userinfo.json'
if not res_path.exists():
    os.mkdir(res_path)



import nonebot
from .data_source import *
from nonebot import on_command
from nonebot.adapters.cqhttp import MessageSegment, GroupMessageEvent, PrivateMessageEvent
from nonebot.typing import T_State
from nonebot.adapters import Bot, Event
from .config import *
from nonebot import require
from .cron_send import *

scheduler = require('nonebot_plugin_apscheduler').scheduler
bugku = on_command("bugku")
start = on_command("start")
init = on_command("init", priority=10)


@init.handle()
async def initialize(bot: Bot, event: Event, state: T_State):
    if not config.InitStatus:
        config.LoginStatus = False
        config.LoginCookies = await get_cookies()
        config.UserInfo = await get_userinfo()
        await init.finish("系统初始化成功，请输入/start执行下一步")
    else:
        await init.finish("系统已初始化完毕，请勿重复初始化")


@start.handle()
async def start_up(bot: Bot, event: GroupMessageEvent, state: T_State):
    user = User(str(event.group_id), 'group')
    if user not in config.UserInfo:
        config.UserInfo.append(user)
        await save_userinfo(config.UserInfo)
        await start.finish("已将本群加入通知列表，请使用/bugku命令即时检测当前房间情况")
    else:
        await start.finish("本群已加入配置，请勿重复初始化")


@start.handle()
async def start_up(bot: Bot, event: PrivateMessageEvent, state: T_State):
    user = User(str(event.user_id), 'private')
    if user not in config.UserInfo:
        config.UserInfo.append(user)
        await save_userinfo(config.UserInfo)
        await start.finish("已将你的QQ加入通知列表，请使用/bugku命令检测当前房间情况")
    else:
        await start.finish("你的QQ已加入配置，请勿重复初始化")


@bugku.args_parser
async def parse(bot: Bot, event: Event, state: T_State):
    state[state["_current_key"]] = str(event.get_message())


@bugku.handle()
async def first_receive(bot: Bot, event: Event, state: T_State):
    # 获取用户原始命令，如：/bugku
    print(state["_prefix"]["raw_command"])
    cookies = await get_cookies()
    if len(cookies) != 0:
        res = await request_detect()
        if '请登录' in res.text:
            scheduler.pause_job('detect_online', jobstore=None)
            await pre_login()
        elif res != '' and res is not None:
            # 正常登录后才能进行工作
            config.LoginStatus = True
            scheduler.resume_job('detect_online')
            await sava_cookies(res)
            await imme_detect_send(res)
    else:
        await pre_login()


@bugku.handle()
async def captcha_receive(bot: Bot, event: Event, state: T_State):
    # 登录事件
    if config.LoginStatus:  # 如果是已登录状态就跳过此handle
        await bugku.finish()
    msg = str(event.get_message()).strip()
    if check_captcha(msg):
        # 符合验证码规范，开始登录
        await login_check(msg)
    else:
        await bugku.reject("验证码格式错误，请重新输入")


async def imme_detect_send(res):
    match_list = await match_data_solve(res)
    if len(match_list) == 0:
        await bugku.finish("Master！暂未侦测到公开房间!")
    else:
        await bugku.send("Master！侦测到公开房间")
        for i in match_list:
            add_data(i)
            await bugku.send("【房间名】 {}\n【赛题】 {}\n【时间】 {}\n【名额】 {}  【入场费】 {}".format(*i))
        await bugku.finish()


@scheduler.scheduled_job('cron', minute='*/1', id='detect_online')
async def cron_detect():
    if not config.LoginStatus:
        return
    bots = nonebot.get_bots()
    for name, bot in bots.items():
        if not isinstance(bot, nonebot.adapters.cqhttp.bot.Bot):
            continue
        res = await request_detect()
        if '请登录' in res.text:
            config.LoginStatus = False
            await bugku.finish()
        match_list = await match_data_solve(res)
        if len(match_list) == 0:
            await bugku.finish()
        else:
            await sava_cookies(res)
            for user in config.UserInfo:
                print("定时任务触发成功")
                tip = "Master！侦测到新的公开房间"
                msg = "【房间名】 {}\n【赛题】 {}\n【时间】 {}\n【名额】 {}  【入场费】 {}"
                msgs = [msg.format(*i) for i in match_list].insert(0, tip)
                send_msgs(bot, user.user, user.user_type, msgs)


async def pre_login():
    await get_captcha()
    await bugku.send(('登录验证码' + MessageSegment.image(f"file:///{img_captcha}")))
    await bugku.pause("Cookie失效，请输入验证码重新登录")  # 暂停handle，准备接收验证码，然后执行下一个handle


async def login_check(captcha):
    res = await request_login(captcha)
    if '登录成功' in res.text:
        config.LoginStatus = True
        await bugku.finish("登录成功，请重新输入指令")
        await sava_cookies(res)
    elif '验证码错误' in res.text:
        await bugku.reject("验证码错误，登录失败，请重新登录")
    else:
        config.LoginStatus = False
        await bugku.finish("登录失败，请检查")


@scheduler.scheduled_job('cron', hour=6, minute=0, id='check_in')
async def cron_check_in():
    res = await check_in()
    msg = ''
    if '签到成功' in res.text:
        msg = '签到成功'
    elif '请登录' in res.text:
        msg = 'Cookies失效，签到失败'
    elif '今天您已经签到过' in res.text:
        return
    bots = nonebot.get_bots()
    for name, bot in bots.items():
        if not isinstance(bot, nonebot.adapters.cqhttp.bot.Bot):
            continue
        send_msgs(bot, config.bugku_own, 'private', [msg])


scheduler.add_job(do_send_msgs, 'interval', seconds=0.3, coalesce=True)



import queue

import nonebot
from nonebot import logger
import time

QUEUE = queue.Queue()
LAST_SEND_TIME = time.time()


async def do_send_msgs():
    global LAST_SEND_TIME
    if time.time() - LAST_SEND_TIME < 1.5:
        return
    if not QUEUE.empty():
        bot, user, user_type, msg, retry_time = QUEUE.get()
        try:
            if user_type == 'group':
                await bot.call_api('send_group_msg', group_id=user, message=msg)
            elif user_type == 'private':
                await bot.call_api('send_private_msg', user_id=user, message=msg)
        except:
            if retry_time > 0:
                QUEUE.put((bot, user, user_type, msg, retry_time - 1))
            else:
                logger.warning('send msg err {}'.format(msg))
        LAST_SEND_TIME = time.time()


def send_msgs(bot: nonebot.Bot, user: str, user_type: str, msgs: list):
    for msg in msgs:
        QUEUE.put((bot, user, user_type, msg, 2))
